@@ -1,7 +1,7 @@
 /*
  * This file is part of the dmidecode project.
  *
- *   (C) 2005-2007 Jean Delvare <khali@linux-fr.org>
+ *   Copyright (C) 2005-2008 Jean Delvare <khali@linux-fr.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -26,8 +26,4 @@ struct dmi_header
 	u8 *data;
 };
 
-const char *dmi_string(struct dmi_header *dm, u8 s);
-void dmi_system_uuid(u8 *p);
-const char *dmi_chassis_type(u8 code);
-const char *dmi_processor_family(u8 code);
-void dmi_processor_frequency(u8 *p);
+const char *dmi_string(const struct dmi_header *dm, u8 s);
